#!/usr/bin/env python

import sys
from dgstudent import *

########## user configurable parameters ########

## choice of solver and O(max number of iterations)
# localsolver:lbfgs,cg,neldermead,leastsq,root
# maxitn:50,10,1000,1000,1000
localsolver = "root" 
maxitn = 500

# infinity
inf = 1e50

########## functions #########

## compute the Gram matrix from the distance matrix
def dist2Gram(EDM):
    n = EDM.shape[0]
    J = np.identity(n) - (1.0/n)*np.ones((n,n))
    B = -0.5 * np.dot(J,np.dot(np.square(EDM), J))
    return B

## perform PCA
def PCA(B,K):
    n = B.shape[0]
    x = factor(B) # returns cols sorted by decreasing eigval
    if K < n:
        # only first K columns (corresp to largest eigvals)
        x = x[:,0:K]
    return x

## the IsoMap method
def IsoMap(G, K):
    ## COMPLETE THIS FUNCTION
    print "dgpisomap: IMPLEMENT ME!"
    return np.zeros((G.V.card,K))

###################### MAIN #####################

## command line expects a _gph.dat file
if len(sys.argv) < 2:
    exit('need file_gph.dat on command line')

## read the instance from file
G = instanceFromFile(sys.argv[1]).graph
instancename = ''.join(os.path.basename(sys.argv[1]).split('_')[:-1])

# set some parameters
n = G.V.card
K = 3
xorg = np.zeros((n,K))
ldeorg = -1
mdeorg = -1

## if given on command line, also read the realization from a _rlz.dat file
if len(sys.argv) >= 3:
    print "dgpisomap: also reading a realization from", sys.argv[2]
    xorg = realizationFromRlzDatFile(G, sys.argv[2])
    ldeorg = LDE(G,xorg)
    mdeorg = MDE(G,xorg)

## IsoMap
xiso = IsoMap(G,K)
ldeiso = LDE(G,xiso)
mdeiso = MDE(G,xiso)
# refine with a local solver
xiso1 = LocalNLPSolver(G, xiso, K, {}, localsolver, maxitn)
ldeiso1 = LDE(G,xiso1)
mdeiso1 = MDE(G,xiso1)

## print results
print "                   ISOMAP        ISOMAP+NLP"
print "dgpisomap:instance,ldeiso,mdeiso,ldeiso1,mdeiso1"
print "{0:s},{1:.2f},{2:.2f},{3:.2f},{4:.2f}".format(instancename,ldeiso,mdeiso,ldeiso1,mdeiso1)

## show results
G.plot(xiso)
G.plot(xiso1)
if ldeorg >= 0:
    G.plot(xorg)
