#!/usr/bin/env python

import sys
from dgstudent import *

########## user configurable parameters ########

## choice of solver and O(max number of iterations)
# localsolver:lbfgs,cg,neldermead,leastsq,root
# maxitn:50,10,1000,1000,1000

#localsolver = "root" 
#maxitn = 1000
localsolver = "lbfgs"
maxitn = 10

# infinity
inf = 1e50

# max iterations of Barvinok's algorithm
barvinokitn = 50

eps = 1e-3

########## functions #########

## compute the Gram matrix from the distance matrix
def dist2Gram(EDM):
    n = EDM.shape[0]
    J = np.identity(n) - (1.0/n)*np.ones((n,n))
    B = -0.5 * np.dot(J,np.dot(np.square(EDM), J))
    return B

## perform PCA
def PCA(B,K):
    n = B.shape[0]
    x = factor(B) # returns cols sorted by decreasing eigval
    if K < n:
        # only first K columns (corresp to largest eigvals)
        x = x[:,0:K]
    return x

## formulate and solve SDP relaxation for DGP
def DGPSDP(G, K):
    ## COMPLETE THIS FUNCTION
    print "DGPSDP: WRITE AN SDP FORMULATION FOR THE DGP"
    t0 = time.time()
    DGP = pic.Problem()
    IdK = cvx.matrix(np.identity(K))

    # decision variables

    # objective function: push-and-pull
    
    # constraints
                
    # SDP constraint
    DGP.add_constraint(( ( IdK & x.T ) // ( x & X ) ) >> 0)
    t1 = time.time()
    # solve and get solution
    DGP.solve()
    t2 = time.time()
    print "dgpsolve: modelling time =", t1-t0, "; solving time =",t2-t1
    Xv = np.array(X.value)
    Xf = factor(Xv)
    objv = DGP.obj_value()
    return (objv, Xf)

## formulate and solve DDP primal approximation from SDP relaxation of DGP
def DGPDDP(G, K):
    ## COMPLETE THIS FUNCTION
    print "DGPDDP: WRITE A DDP FORMULATION FOR THE DGP"
    t0 = time.time()
    DGP = pic.Problem()
    IdK = cvx.matrix(np.identity(K))

    ## decision variables

    ## objective function: push-and-pull
    
    ## constraints

    # distance

    # diagonal dominance

    # sandwich

    t1 = time.time()
    # solve and get solution
    DGP.solve()
    t2 = time.time()
    print "dgpsolve: modelling time =", t1-t0, "; solving time =",t2-t1
    Xv = np.array(X.value)
    Xf = factor(Xv)
    objv = DGP.obj_value()
    return (objv, Xf)


## Barvinok's "naive algorithm"
def Barvinok(G,T,K):
    ## COMPLETE THIS FUNCTION
    print "Barvinok: given factor T of SDP sol. X, implement Barvinok's naive algorithm"
    t0 = time.time()
    (n,n) = T.shape
    x = np.zeros((n,K))
    ## work here, fill x
    t1 = time.time()
    print "dgpsolve: Barvinok time =", t1-t0
    return x
    
###################### MAIN #####################

## command line expects a _gph.dat file
if len(sys.argv) < 4:
    print "cmd line is: file_gph.dat <sdp|ddp> <pca|barvinok> [file_rlz.dat]"
    exit()

## read the instance from file
G = instanceFromFile(sys.argv[1]).graph
instancename = ''.join(os.path.basename(sys.argv[1]).split('_')[:-1])

# Formulation: default is SDP
formulationmethod = 0  # SDP
#formulationmethod = 1  # DDP
if sys.argv[2] == "ddp":
    formulationmethod = 1  # DDP

# dimension reduction method (from n to K)
dimreductionmethod = 0 #PCA
#dimreductionmethod = 1 #Barvinok
#dimreductionmethod = 2 # first K comps
if sys.argv[3] == "pca":
    dimreductionmethod = 0
elif sys.argv[3] == "barvinok":
    dimreductionmethod = 1
else:
    dimreductionmethod = 2

# set some parameters
n = G.V.card
K = 3
xorg = np.zeros((n,K))
ldeorg = -1
mdeorg = -1

## if given on command line, also read the realization from a _rlz.dat file
if len(sys.argv) >= 5:
    print "dgpsolve: also reading a realization from", sys.argv[4]
    xorg = realizationFromRlzDatFile(G, sys.argv[4])
    ldeorg = LDE(G,xorg)
    mdeorg = MDE(G,xorg)

if formulationmethod == 0:
    # call SDP
    (obj,X) = DGPSDP(G,K)
else:
    # call DDP
    (obj,X) = DGPDDP(G,K)
    
print "|V| =",n, "rk(X) =", np.linalg.matrix_rank(X)
ldeX = LDE(G,X)
mdeX = MDE(G,X)

if dimreductionmethod == 0:
    # call PCA
    xsdp = PCA(X,K)
elif dimreductionmethod == 1:
    # call Barvinok
    xsdp = Barvinok(G,X,K)
else:
    # first K columns
    xsdp = X[:,0:K]
    
ldesdp = LDE(G,xsdp)
mdesdp = MDE(G,xsdp)

# refine with a local solver
xsdp1 = LocalNLPSolver(G, xsdp, K, {}, localsolver, maxitn)
ldesdp1 = LDE(G,xsdp1)
mdesdp1 = MDE(G,xsdp1)

## print results
if formulationmethod == 0:
    if dimreductionmethod == 0:
        print "dgpsolve:         SDP     +PCA    +LOCNLP"
    elif dimreductionmethod == 1:
        print "dgpsolve:         SDP     +Brvnk  +LOCNLP"
    else:
        print "dgpsolve:         SDP     +dimRed +LOCNLP"
else:
    if dimreductionmethod == 0:
        print "dgpsolve:         DDP     +PCA    +LOCNLP"
    elif dimreductionmethod == 1:
        print "dgpsolve:         DDP     +Brvnk  +LOCNLP"
    else:
        print "dgpsolve:         DDP     +dimRed +LOCNLP"
    
print "dgpsolve:instance,lde,mde,lde,mde,lde,mde"
print "{0:s},{1:.2f},{2:.2f},{3:.2f},{4:.2f},{5:.2f},{6:.2f}".format(instancename,ldeX,mdeX,ldesdp,mdesdp,ldesdp1,mdesdp1)

# show results
G.plot(xsdp)
G.plot(xsdp1)
if ldeorg >= 0:
    G.plot(xorg)
